﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace CleanClickOnceTempFiles
{
	public partial class frmMain : Form
	{
		private string rootPath;
		private string appPath;
		private string tempPath;
		private static readonly string PROC_NAME = "dfsvc";

		public frmMain()
		{
			InitializeComponent();

			initializeControls();
		}

		private void initializeControls()
		{
			rootPath = Directory.GetParent(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData)).FullName;

			appPath = Path.Combine(rootPath, "Apps");

			tempPath = Path.Combine(Path.GetTempPath(), "Deployment");

			txtCurrentUser.Text = Environment.UserName;
			txtRootPath.Text = rootPath;
			txtAppPath.Text = appPath;
			txtTemp.Text = tempPath;

			refreshCounts();
		}

		private void btnReset_Click(object sender, EventArgs e)
		{
			refreshCounts();
		}
	
		private void refreshCounts()
		{
			this.tempPathCnt.Text = getPathFileCountString(tempPath);
			this.UserPathCnt.Text = getPathFileCountString(appPath);
			this.procCount.Text = getProcCount();

			flashWindow(Color.Green);
		}

		private string getProcCount()
		{
			Process[] procs = Process.GetProcessesByName(PROC_NAME);

			return string.Format("{0} instances of {1}.exe, 1st PID={2}",
									procs.Length.ToString(),
									PROC_NAME,
									procs.Length > 0 ? procs[0].Id.ToString() : "na");
		}

		private string getPathFileCountString(string path)
		{
			return string.Format("{0} Contains {1} files.", Path.GetFileName(path), countChildFiles(path, 0));
		}

		private int countChildFiles(string path, int curCount)
		{
			int i = curCount;

			if (!Directory.Exists(path)) return i;

			foreach (string s in Directory.GetFiles(path, "*.*"))
			{
				i++;
			}

			foreach (string dir in Directory.GetDirectories(path))
				i += countChildFiles(dir, 0);

			return i;
		}

		private void killProcessToolStripMenuItem_Click(object sender, EventArgs e)
		{
			int cnt = 0;
			int err = 0;

			foreach (Process proc in Process.GetProcessesByName(PROC_NAME))
			{
				try
				{
					proc.Kill();
					cnt++;
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message);
					err++;
				}
			}

			if (cnt > 0)
			{
				//wait for Process Collection to refresh:
				System.Threading.Thread.Sleep(1000);

				refreshCounts();
			}
			
			MessageBox.Show(string.Format("{0} killed, {1} errors.", cnt, err));
		}

		private void btnClearUser_Click(object sender, EventArgs e)
		{
			zapDir(appPath);
		}

		private void btnCleanTemp_Click(object sender, EventArgs e)
		{
			zapDir(tempPath);
		}

		private void zapDir(string path)
		{
			if (!Directory.Exists(path))
			{
				flashWindow(Color.Red);
				return;
			}

			killAllChildFiles(path);
			refreshCounts();
			Process.Start(Directory.GetParent(path).ToString());
		}

		private void killAllChildFiles(string path)
		{
			foreach (string s in Directory.GetFiles(path, "*.*"))
				File.Delete(Path.Combine(path, s));

			foreach (string subdir in Directory.GetDirectories(path))
				killAllChildFiles(subdir);

			Directory.Delete(path);
		}

		private void flashWindow(Color flashColor)
		{
			Color bk = this.BackColor;
			
			for (int r = 1; r <= 2; r++)
			{
				this.BackColor = flashColor;
				this.Invalidate();
				this.Refresh();
				Thread.Sleep(150);
				
				this.BackColor = bk;
				this.Invalidate();
				this.Refresh();
				Thread.Sleep(150);
			}
		}

	}
}